/* eslint-disable no-magic-numbers */
import {
  ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
  ELEMENT_SEARCH_RESULT_MOCK_REACTION,
} from '@/models/mocks/elementSearchResultMock';
import { apiPath } from '@/redux/apiPath';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { waitFor } from '@testing-library/react';
import { HttpStatusCode } from 'axios';
import { MapBrowserEvent } from 'ol';
import * as handleDataReset from './handleDataReset';
import * as handleSearchResultAction from './handleSearchResultAction';
import { onMapSingleClick } from './onMapSingleClick';

jest.mock('./handleSearchResultAction', () => ({
  __esModule: true,
  ...jest.requireActual('./handleSearchResultAction'),
}));
jest.mock('./handleDataReset', () => ({
  __esModule: true,
  ...jest.requireActual('./handleDataReset'),
}));

const mockedAxiosOldClient = mockNetworkResponse();

const handleSearchResultActionSpy = jest.spyOn(
  handleSearchResultAction,
  'handleSearchResultAction',
);
const handleDataResetSpy = jest.spyOn(handleDataReset, 'handleDataReset');

const getEvent = (coordinate: MapBrowserEvent<UIEvent>['coordinate']): MapBrowserEvent<UIEvent> =>
  ({
    coordinate,
  }) as unknown as MapBrowserEvent<UIEvent>;

describe('onMapSingleClick - util', () => {
  beforeEach(() => {
    jest.clearAllMocks();
    jest.resetAllMocks();
  });

  describe('when always', () => {
    const { store } = getReduxStoreWithActionsListener();
    const { dispatch } = store;
    const modelId = 1000;
    const mapSize = {
      width: 90,
      height: 90,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const handler = onMapSingleClick(mapSize, modelId, dispatch);
    const coordinate = [90, 90];
    const event = getEvent(coordinate);

    it('should fire data reset handler', async () => {
      await handler(event);
      expect(handleDataResetSpy).toBeCalled();
    });
  });

  describe('when searchResults are undefined', () => {
    const { store } = getReduxStoreWithActionsListener();
    const { dispatch } = store;
    const modelId = 1000;
    const mapSize = {
      width: 90,
      height: 90,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const handler = onMapSingleClick(mapSize, modelId, dispatch);
    const coordinate = [90, 90];
    const point = { x: 180.0008084837557, y: 179.99919151624428 };
    const event = getEvent(coordinate);

    mockedAxiosOldClient
      .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
      .reply(HttpStatusCode.Ok, undefined);

    it('does not fire search result action', async () => {
      await handler(event);
      expect(handleSearchResultActionSpy).not.toBeCalled();
    });
  });

  describe('when searchResults are empty', () => {
    const { store } = getReduxStoreWithActionsListener();
    const { dispatch } = store;

    const modelId = 1000;
    const mapSize = {
      width: 180,
      height: 180,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };

    const handler = onMapSingleClick(mapSize, modelId, dispatch);
    const coordinate = [180, 180];
    const point = { x: 360.0032339350228, y: 359.9967660649771 };
    const event = getEvent(coordinate);

    mockedAxiosOldClient
      .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
      .reply(HttpStatusCode.Ok, []);

    it('does not fire search result action', async () => {
      await handler(event);
      expect(handleSearchResultActionSpy).not.toBeCalled();
    });
  });

  describe('when searchResults are valid', () => {
    describe('when results type is ALIAS', () => {
      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;
      const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_ALIAS;
      const mapSize = {
        width: 270,
        height: 270,
        tileSize: 256,
        minZoom: 2,
        maxZoom: 9,
      };
      const coordinate = [270, 270];
      const point = { x: 540.0072763538013, y: 539.9927236461986 };
      const event = getEvent(coordinate);

      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_ALIAS]);

      it('does fire search result action handler', async () => {
        const handler = onMapSingleClick(mapSize, modelId, dispatch);
        await handler(event);
        await waitFor(() => expect(handleSearchResultActionSpy).toBeCalled());
      });
    });

    describe('when results type is REACTION', () => {
      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;
      const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_REACTION;
      const mapSize = {
        width: 0,
        height: 0,
        tileSize: 256,
        minZoom: 2,
        maxZoom: 9,
      };
      const coordinate = [0, 0];
      const point = {
        x: 0,
        y: 0,
      };
      const event = getEvent(coordinate);

      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_REACTION]);

      it('does fire search result action - handle reaction', async () => {
        const handler = onMapSingleClick(mapSize, modelId, dispatch);
        await handler(event);
        await waitFor(() => expect(handleSearchResultActionSpy).toBeCalled());
      });
    });
  });
});
