import { mirnasFixture } from '@/models/fixtures/mirnasFixture';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { HttpStatusCode } from 'axios';
import { apiPath } from '@/redux/apiPath';
import { DEFAULT_ERROR } from '@/constants/errors';
import { getMirnas } from './mirnas.thunks';
import mirnasReducer from './mirnas.slice';
import { MirnasState } from './mirnas.types';

const mockedAxiosClient = mockNetworkResponse();
const SEARCH_QUERY = 'hsa-miR-302b-3p';

const INITIAL_STATE: MirnasState = {
  data: [],
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('mirnas reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<MirnasState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('mirnas', mirnasReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(mirnasReducer(undefined, action)).toEqual(INITIAL_STATE);
  });

  it('should update store after succesfull getMirnas query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getMirnasStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, mirnasFixture);

    const { type } = await store.dispatch(getMirnas(SEARCH_QUERY));
    const { data } = store.getState().mirnas;
    const mirnasWithSearchElement = data.find(mirna => mirna.searchQueryElement === SEARCH_QUERY);

    expect(type).toBe('project/getMirnas/fulfilled');
    expect(mirnasWithSearchElement).toEqual({
      searchQueryElement: SEARCH_QUERY,
      data: mirnasFixture,
      loading: 'succeeded',
      error: DEFAULT_ERROR,
    });
  });

  it('should update store after failed getMirnas query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getMirnasStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.NotFound, []);

    const { type } = await store.dispatch(getMirnas(SEARCH_QUERY));
    const { data } = store.getState().mirnas;
    const mirnasWithSearchElement = data.find(mirna => mirna.searchQueryElement === SEARCH_QUERY);

    expect(type).toBe('project/getMirnas/rejected');
    expect(mirnasWithSearchElement).toEqual({
      searchQueryElement: SEARCH_QUERY,
      data: undefined,
      loading: 'failed',
      error: DEFAULT_ERROR,
    });
  });

  it('should update store on loading getMirnas query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getMirnasStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, mirnasFixture);

    const mirnasPromise = store.dispatch(getMirnas(SEARCH_QUERY));

    const { data } = store.getState().mirnas;
    const mirnasWithSearchElement = data.find(mirna => mirna.searchQueryElement === SEARCH_QUERY);

    expect(mirnasWithSearchElement).toEqual({
      searchQueryElement: SEARCH_QUERY,
      data: undefined,
      loading: 'pending',
      error: DEFAULT_ERROR,
    });

    mirnasPromise.then(() => {
      const { data: dataPromiseFulfilled } = store.getState().mirnas;

      const mirnasWithSearchElementFulfilled = dataPromiseFulfilled.find(
        mirna => mirna.searchQueryElement === SEARCH_QUERY,
      );

      expect(mirnasWithSearchElementFulfilled).toEqual({
        searchQueryElement: SEARCH_QUERY,
        data: mirnasFixture,
        loading: 'succeeded',
        error: DEFAULT_ERROR,
      });
    });
  });
});
